unit OComps;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

const Margin = 5;

type
  TCmMouseEnter = record
    Msg    : Cardinal;
    Unused : Integer;
    Sender : TControl;
    Result : LongInt;
  end;
  TCmMouseLeave = TCmMouseEnter;

  TMouseEnter = procedure (Sender: TControl; var Message: TCMMouseEnter) of object;
  TMouseLeave = procedure (Sender: TControl; var Message: TCMMouseLeave) of object;

  // TOButton control definition...
  TOButton = class(TCustomControl)
  private
    FBorderColor: TColor;
    FActiveBorderColor: TColor;
    FLed: TBitmap;
    FRoundSize: integer;

    FDefault: Boolean;
    FCancel: Boolean;
    FActive: Boolean;
    FModalResult: TModalResult;

    FMouseEnter: TMouseEnter;
    FMouseLeave: TMouseLeave;
    HasMouse, MouseIsDown, Focused: boolean;

    procedure SetBorderColor(index: integer; value: TColor);
    procedure SetDefault(Value: Boolean);
    procedure SetRoundSize(value: Integer);

    procedure WMMouseMove(var Message: TWMMouseMove); message WM_MOUSEMOVE;
    procedure CmDialogKey(var Message: TCMDialogKey); message CM_DIALOGKEY;
    procedure CmDialogChar(var Message: TCMDialogChar); message CM_DIALOGCHAR;
    procedure CmEnabledChanged(var Message: TWmNoParams); message CM_ENABLEDCHANGED;
    procedure CmFocusChanged(var Message: TCMFocusChanged); message CM_FOCUSCHANGED;
    procedure CmMouseEnter(var Message: TCmMouseEnter); message CM_MOUSEENTER;
    procedure CmMouseLeave(var Message: TCmMouseLeave); message CM_MOUSELEAVE;
    procedure CmParentColorChanged(var Message: TWMNoParams); message CM_PARENTCOLORCHANGED;
    procedure CmTextChanged(var Message: TWmNoParams); message CM_TEXTCHANGED;
    procedure CmVisibleChanged(var Message: TWmNoParams); message CM_VISIBLECHANGED;
    procedure CnCommand(var Message: TWMCommand); message CN_COMMAND;
  protected
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyUp(var Key: Word; Shift: TShiftState); override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure DoEnter; override;
    procedure DoExit; override;
    procedure Paint; override;
    procedure DrawBorder;
    procedure DrawFace;
    procedure DrawLed;
    procedure DrawCaption;
    procedure DrawFocus;
    procedure SetButtonStyle(ADefault: Boolean); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property ActiveBorderColor: TColor index 0 read FActiveBorderColor write SetBorderColor default clGray;
    property BorderColor: TColor index 1 read FBorderColor write SetBorderColor default clSilver;
    property Cancel: Boolean read FCancel write FCancel default False;
    property Caption;
    property Color;
    property Default: Boolean read FDefault write SetDefault default False;
    property Enabled;
    property Font;
    property Hint;
    property ModalResult: TModalResult read FModalResult write FModalResult default 0;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property RoundSize: Integer read FRoundSize write SetRoundSize default 20;
    property ShowHint;
    property TabOrder;
    property TabStop default True;
    property Visible;
    property OnClick;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseEnter: TMouseEnter read FMouseEnter write FMouseEnter;
    property OnMouseLeave: TMouseLeave read FMouseLeave write FMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
  end;

  // TOLabel control definition...
  TLEDType = (ltDefault, ltUp, ltDown);
  TMargin = 0..9;
  TTabStop = 1..8;

  TOLabel = class(TCustomControl)
  private
    FLed: TBitmap;
    FLedType: TLedType;

    FAlignment: TAlignment;
    FActive: boolean;
    FCancel: boolean;
    FDefault: boolean;
    FMarginLeft: TMargin;
    FMarginRight: TMargin;
    FModalResult: TModalResult;
    FTabSize: TTabStop;

    HasMouse, MouseIsDown, Focused: boolean;
    FMouseEnter: TMouseEnter;
    FMouseLeave: TMouseLeave;
    FCaptRect: TRect;

    procedure SetAlignment(Value: TAlignment);
    procedure SetDefault(Value: Boolean);
    procedure SetLedType(Value: TLedType);
    procedure SetMargin(index: integer; Value: TMargin);
    procedure SetTabSize(Value: TTabStop);

    procedure WMMouseMove(var Message: TWMMouseMove); message WM_MOUSEMOVE;
    procedure CmDialogKey(var Message: TCMDialogKey); message CM_DIALOGKEY;
    procedure CmDialogChar(var Message: TCMDialogChar); message CM_DIALOGCHAR;
    procedure CmEnabledChanged(var Message: TWmNoParams); message CM_ENABLEDCHANGED;
    procedure CmFocusChanged(var Message: TCMFocusChanged); message CM_FOCUSCHANGED;
    procedure CmMouseEnter(var Message: TCmMouseEnter); message CM_MOUSEENTER;
    procedure CmMouseLeave(var Message: TCmMouseLeave); message CM_MOUSELEAVE;
    procedure CmParentColorChanged(var Message: TWMNoParams); message CM_PARENTCOLORCHANGED;
    procedure CmParentFontChanged(var Message: TWMNoParams); message CM_PARENTFONTCHANGED;
    procedure CmTextChanged(var Message: TWmNoParams); message CM_TEXTCHANGED;
    procedure CmVisibleChanged(var Message: TWmNoParams); message CM_VISIBLECHANGED;
    procedure CnCommand(var Message: TWMCommand); message CN_COMMAND;
  protected
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyUp(var Key: Word; Shift: TShiftState); override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure DoEnter; override;
    procedure DoExit; override;
    procedure Paint; override;
    procedure DrawLed;
    procedure DrawFocus;
    procedure DrawCaption;
    procedure SetButtonStyle(ADefault: Boolean); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Alignment: TAlignment read FAlignment write SetAlignment default taLeftJustify;
    property Cancel: Boolean read FCancel write FCancel default False;
    property Caption;
    property Default: Boolean read FDefault write SetDefault default False;
    property Enabled;
    property Font;
    property Hint;
    property LedType: TLedType read FLedType write SetLedType default ltDefault;
    property MarginLeft: TMargin index 0 read FMarginLeft write SetMargin default 1;
    property MarginRight: TMargin index 1 read FMarginRight write SetMargin default 1;
    property ModalResult: TModalResult read FModalResult write FModalResult default 0;
    property ParentFont;
    property ParentShowHint;
    property ShowHint;
    property TabSize: TTabStop read FTabSize write SetTabSize default 3;
    property TabOrder;
    property TabStop default True;
    property Visible;
    property OnClick;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseEnter: TMouseEnter read FMouseEnter write FMouseEnter;
    property OnMouseLeave: TMouseLeave read FMouseLeave write FMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
  end;

implementation

{$R OComps.res}

// Beginning of TOButton code...
procedure TOButton.DrawBorder;
var
  BtnRect: TRect;
begin
  with Canvas do
  begin
    with Pen do
    begin
      Style := psInsideFrame;
      if HasMouse then
      begin
        Color := ActiveBorderColor;
        Width := 2;
      end
      else
      begin
        Color := BorderColor;
        Width := 1;
      end;
    end;
    Brush.Color := Self.Color;
    BtnRect := Rect(0, 0, ClientWidth, ClientHeight);
    RoundRect(BtnRect.Left, BtnRect.Top, BtnRect.Right, BtnRect.Bottom, RoundSize, RoundSize);
  end;
end;

procedure TOButton.DrawCaption;
var
  CaptRect: TRect;
  CaptOrg: TPoint;
  CaptWidth: integer;
begin
  with Canvas do
  begin
    Font := Self.Font;
    Brush.Style := bsClear;
    if not Enabled then Font.Color := clGray;
    if Default then Font.Style := [fsBold];
    if Pos('&', Caption) > 0 then CaptWidth := (TextWidth(Caption) - TextWidth('&')) else CaptWidth := TextWidth(Caption);
    CaptOrg.X := (ClientWidth - CaptWidth) div 2;
    CaptOrg.Y := (Height-TextHeight(Caption)) div 2;
    CaptRect := Rect(CaptOrg.X, CaptOrg.Y, ClientWidth-CaptOrg.X, CaptOrg.Y+TextHeight(Caption));
    if HasMouse and MouseIsDown then OffsetRect(CaptRect,1,1);
    DrawText(Handle, PChar(Caption), Length(Caption), CaptRect, DT_LEFT);
  end;
end;

procedure TOButton.DrawFace;
var
  Border: TColor;
begin
  with canvas do
  begin
    Brush.Color := Parent.Brush.Color;
    if HasMouse then Border := ActiveBorderColor else Border := BorderColor;
    FloodFill(1, 1, Border, fsBorder);
    FloodFill(1, Height-1, Border, fsBorder);
    FloodFill(Width-1, 1, Border, fsBorder);
    FloodFill(Width-1, Height-1, Border, fsBorder);
  end;
end;

procedure TOButton.DrawLed;
var
  Dest, Src: TRect;
begin
  with Canvas do
  begin
    if Enabled then
    begin
      if HasMouse then
        if MouseIsDown then
          FLed.Handle := LoadBitmap(hInstance, 'DOWN')
        else
          FLed.Handle := LoadBitmap(hInstance, 'ON')
      else
        FLed.Handle := LoadBitmap(hInstance, 'OFF');
    end
    else
      FLed.Handle := LoadBitmap(hInstance, 'DISABLED');

    Src := Rect(0, 0, FLed.Width, FLed.Height);
    Dest := Rect(Margin, ((Height - FLed.Height) div 2), Margin+FLed.Width, ((Height - FLed.Height) div 2)+FLed.Height);
    if HasMouse and MouseIsDown then OffsetRect(Dest,1,1);
    Brush.Color := Self.Color;
    Brush.Style := bsClear;
    BrushCopy(Dest, FLed, Src, clBlack);
  end;
end;

procedure TOButton.DrawFocus;
var
  CaptHeight, CaptWidth: integer;
  FocusOrg: TPoint;
begin
  if Focused then
  begin
    with Canvas do
    begin
      Font := Self.Font;
      Brush.Style := bsClear;
      if Default then Font.Style := [fsBold];
      CaptHeight := TextHeight(Caption)+2;
      CaptWidth := TextWidth(Caption)+(Margin)+1;
      if Pos('&', Caption) > 0 then CaptWidth := CaptWidth - TextWidth('&');
      with Pen do
      begin
        Mode := pmMaskPenNot;
        Style := psDot;
        Width := 1;
        Color := clBlack;
        Mode := pmCopy;
      end;
      FocusOrg.X := ((ClientWidth - CaptWidth) div 2);
      FocusOrg.Y := (Height-CaptHeight) div 2;
      DrawFocusRect(Rect(FocusOrg.X, FocusOrg.Y, FocusOrg.X+CaptWidth, CaptHeight+FocusOrg.Y));
    end;
  end;
end;

procedure TOButton.SetBorderColor(index: integer; value: TColor);
begin
  case index of
    0: begin
         if value <> FActiveBorderColor then
            FActiveBorderColor := value;
        end;
    1: begin
         if value <> FBorderColor then
            FBorderColor := value;
       end;
  end;
  Repaint;
end;

procedure TOButton.SetButtonStyle(ADefault: Boolean);
const
  BS_MASK = $000F;
var
  Style: Word;
begin
  if HandleAllocated then
  begin
    if ADefault then Style := BS_DEFPUSHBUTTON else Style := BS_PUSHBUTTON;
    if GetWindowLong(Handle, GWL_STYLE) and BS_MASK <> Style then
      SendMessage(Handle, BM_SETSTYLE, Style, 1);
  end;
end;

procedure TOButton.SetDefault(Value: Boolean);
var
  Form: TCustomForm;
begin
  FDefault := Value;
  if HandleAllocated then
  begin
    Form := GetParentForm(Self);
    if Form <> nil then
      Form.Perform(CM_FOCUSCHANGED, 0, Longint(Form.ActiveControl));
  end;
  Repaint;
end;

procedure TOButton.SetRoundSize(value: Integer);
begin
  if value <> FRoundSize then
  begin
    FRoundSize := value;
    Repaint;
  end;
end;

procedure TOButton.Paint;
begin
  inherited Paint;
  if (csDesigning in Componentstate) or Visible then
  begin
    DrawBorder;
    if not ParentColor then DrawFace;
    DrawLed;
    DrawFocus;
    DrawCaption;
  end;
end;

procedure TOButton.WMMouseMove(var Message: TWMMouseMove);
var
  CurrentPoint: TPoint;
begin
  CurrentPoint.X := Message.XPos;
  CurrentPoint.Y := Message.YPos;
  if not HasMouse and PtInRect(GetClientRect, CurrentPoint) then
  begin
    HasMouse := true;
    Repaint;
  end;
  inherited;
end;

procedure TOButton.CMDialogKey(var Message: TCMDialogKey);
begin
  with Message do
    if (((CharCode = VK_RETURN) and FActive) or ((CharCode = VK_ESCAPE) and FCancel))
       and (KeyDataToShiftState(Message.KeyData) = []) and CanFocus then
    begin
       Click;
       Result := 1;
    end
    else
      inherited;
end;

procedure TOButton.CMDialogChar(var Message: TCMDialogChar);
begin
  with Message do
    if IsAccel(CharCode, Caption) and CanFocus then
    begin
      Click;
      Result := 1;
    end
    else
      inherited;
end;

procedure TOButton.CmEnabledChanged(var Message: TWmNoParams);
begin
  inherited;
  Repaint;
end;

procedure TOButton.CMFocusChanged(var Message: TCMFocusChanged);
begin
  with Message do
    if Sender is TOButton then
      FActive := Sender = Self
    else
      FActive := FDefault;
  SetButtonStyle(FActive);
  inherited;
end;

procedure TOButton.CmMouseEnter(var Message: TCmMouseEnter);
begin
  inherited;
  if Enabled then
  begin
    HasMouse := true;
    Repaint;
  end;
  if Assigned(FMouseEnter) then FMouseEnter(Self, Message);
end;

procedure TOButton.CmMouseLeave(var Message: TCmMouseLeave);
begin
  inherited;
  if Enabled then
  begin
    HasMouse := false;
    Repaint;
  end;
  if Assigned(FMouseLeave) then FMouseLeave(Self, Message);
end;

procedure TOButton.CmParentColorChanged(var Message: TWMNoParams);
begin
  inherited;
  RecreateWnd;
end;

procedure TOButton.CmTextChanged(var Message: TWmNoParams);
begin
  inherited;
  if not (csDesigning in ComponentState) then ControlStyle := ControlStyle - [csOpaque];
  Repaint;
  if not (csDesigning in ComponentState) then ControlStyle := ControlStyle + [csOpaque];
end;

procedure TOButton.CmVisibleChanged(var Message: TWmNoParams);
begin
  inherited;
  Repaint;
end;

procedure TOButton.CNCommand(var Message: TWMCommand);
begin
  if Message.NotifyCode = BN_CLICKED then Click;
end;

procedure TOButton.DoEnter;
begin
  inherited DoEnter;
  Focused := true;
  Repaint;
end;

procedure TOButton.DoExit;
begin
  inherited DoExit;
  Focused := false;
  Repaint;
end;

procedure TOButton.KeyDown(var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_SPACE) then
  begin
    HasMouse := true;
    MouseIsDown := true;
    Repaint;
  end;
  inherited KeyDown(Key, Shift);
end;

procedure TOButton.KeyUp(var Key: Word; Shift: TShiftState);
var
  CurPos: TPoint;
begin
  inherited KeyUp(Key, Shift);
  if (KEY = VK_SPACE) then
  begin
    GetCursorPos(CurPos);
    CurPos := ScreenToClient(CurPos);
    if PtInRect(GetClientRect, CurPos) then HasMouse := true else HasMouse := false;
    MouseIsDown := false;
    Repaint;
    Click;
  end;       
  
end;

procedure TOButton.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if (Button = mbLeft) and Enabled then
  begin
    SetFocus;
    MouseIsDown := true;
    Repaint;
    inherited MouseDown(Button, Shift, X, Y);
  end;
end;

procedure TOButton.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if (Button = mbLeft) and Enabled then
  begin
    MouseIsDown := false;
    Repaint;
    inherited MouseUp(Button, Shift, X, Y);
  end;
end;

constructor TOButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FLed := TBitmap.Create;
  FLed.Handle := LoadBitmap(hInstance, 'OFF');
  if not (csDesigning in ComponentState) then controlstyle := controlstyle + [csOpaque, csSetCaption];
  SetBounds(0, 0, 85, 23);
  ActiveBorderColor := clGray;
  BorderColor := clSilver;
  RoundSize := 20;
  TabStop := true;
end;

destructor TOButton.Destroy;
begin
  FLed.Free;
  inherited Destroy;
end;

// Beginning of TOLabel code...
procedure TOLabel.SetButtonStyle(ADefault: Boolean);
const
  BS_MASK = $000F;
var
  Style: Word;
begin
  if HandleAllocated then
  begin
    if ADefault then Style := BS_DEFPUSHBUTTON else Style := BS_PUSHBUTTON;
    if GetWindowLong(Handle, GWL_STYLE) and BS_MASK <> Style then
      SendMessage(Handle, BM_SETSTYLE, Style, 1);
  end;
end;

procedure TOLabel.SetAlignment(Value: TAlignment);
begin
  if Value <> FAlignment then
  begin
    FAlignment := Value;
    Repaint;
  end;
end;

procedure TOLabel.SetDefault(Value: Boolean);
var
  Form: TCustomForm;
begin
  FDefault := Value;
  if HandleAllocated then
  begin
    Form := GetParentForm(Self);
    if Form <> nil then
      Form.Perform(CM_FOCUSCHANGED, 0, Longint(Form.ActiveControl));
  end;
  Repaint;
end;

procedure TOLabel.SetLedType(Value: TLedType);
begin
  if value <> FLedType then
  begin
    FLedType := value;
    RecreateWnd;
  end;
end;

procedure TOLabel.SetMargin(index: integer; Value: TMargin);
begin
  case index of
    0: if Value <> FMarginLeft then
       begin
         FMarginLeft := Value;
         Repaint;
       end;
    1: if Value <> FMarginRight then
       begin
         FMarginRight := Value;
         Repaint;
       end;
  end;
end;

procedure TOLabel.SetTabSize(Value: TTabStop);
begin
  if Value <> FTabSize then
  begin
    FTabSize := Value;
    Repaint;
  end;
end;

procedure TOLabel.WMMouseMove(var Message: TWMMouseMove);
var
  CurrentPoint: TPoint;
begin
  CurrentPoint.X := Message.XPos;
  CurrentPoint.Y := Message.YPos;
  if not HasMouse and PtInRect(GetClientRect, CurrentPoint) then
  begin
    HasMouse := true;
    Repaint;
  end;
  inherited;
end;

procedure TOLabel.CMDialogKey(var Message: TCMDialogKey);
begin
  with Message do
    if (((CharCode = VK_RETURN) and FActive) or ((CharCode = VK_ESCAPE) and FCancel))
       and (KeyDataToShiftState(Message.KeyData) = []) and CanFocus then
    begin
       Click;
       Result := 1;
    end
    else
      inherited;
end;

procedure TOLabel.CMDialogChar(var Message: TCMDialogChar);
begin
  with Message do
    if IsAccel(CharCode, Caption) and CanFocus then
    begin
      Click;
      Result := 1;
    end
    else
      inherited;
end;

procedure TOLabel.CMFocusChanged(var Message: TCMFocusChanged);
begin
  with Message do
    if Sender is TOLabel then
      FActive := Sender = Self
    else
      FActive := FDefault;
  SetButtonStyle(FActive);
  inherited;
end;

procedure TOLabel.CmEnabledChanged(var Message: TWmNoParams);
begin
  inherited;
  Repaint;
end;

procedure TOLabel.CmMouseEnter(var Message: TCmMouseEnter);
begin
  inherited;
  if Enabled then
  begin
    HasMouse := true;
    Repaint;
  end;
  if Assigned(FMouseEnter) then FMouseEnter(Self, Message);
end;

procedure TOLabel.CmMouseLeave(var Message: TCmMouseLeave);
begin
  inherited;
  if Enabled then
  begin
    HasMouse := false;
    Repaint;
  end;
  if Assigned(FMouseLeave) then FMouseLeave(Self, Message);
end;

procedure TOLabel.CmParentColorChanged(var Message: TWMNoParams);
begin
  inherited;
  RecreateWnd;
end;

procedure TOLabel.CmParentFontChanged(var Message: TWMNoParams);
begin
  inherited;
  RecreateWnd;
end;

procedure TOLabel.CmTextChanged(var Message: TWmNoParams);
begin
  inherited;
  if not (csDesigning in ComponentState) then ControlStyle := ControlStyle - [csOpaque];
  Repaint;
  if not (csDesigning in ComponentState) then ControlStyle := ControlStyle + [csOpaque];
end;

procedure TOLabel.CmVisibleChanged(var Message: TWmNoParams);
begin
  inherited;
  Repaint;
end;

procedure TOLabel.CNCommand(var Message: TWMCommand);
begin
  if Message.NotifyCode = BN_CLICKED then Click;
end;

procedure TOLabel.DrawLed;
var
  Dest, Src: TRect;
begin
  with Canvas do
  begin
    case LedType of
         ltDefault: begin
                    if Enabled then
                       if HasMouse then
                          if MouseIsDown then
                             FLed.Handle := LoadBitmap(hInstance, 'DOWNRND')
                          else
                             FLed.Handle := LoadBitmap(hInstance, 'ONRND')
                       else
                       FLed.Handle := LoadBitmap(hInstance, 'OFFRND')
                    else
                      FLed.Handle := LoadBitmap(hInstance, 'DISRND');
                   end;
        ltUp:      begin
                    if Enabled then
                       if HasMouse then
                          if MouseIsDown then
                             FLed.Handle := LoadBitmap(hInstance, 'DOWN3UP')
                          else
                             FLed.Handle := LoadBitmap(hInstance, 'ON3UP')
                       else
                       FLed.Handle := LoadBitmap(hInstance, 'OFF3UP')
                    else
                      FLed.Handle := LoadBitmap(hInstance, 'DIS3UP');
                   end;
        ltDown:    begin
                    if Enabled then
                       if HasMouse then
                          if MouseIsDown then
                             FLed.Handle := LoadBitmap(hInstance, 'DOWN3DWN')
                          else
                             FLed.Handle := LoadBitmap(hInstance, 'ON3DWN')
                       else
                       FLed.Handle := LoadBitmap(hInstance, 'OFF3DWN')
                    else
                      FLed.Handle := LoadBitmap(hInstance, 'DIS3DWN');
                   end;
      end;
    Src := Rect(0, 0, FLed.Width, FLed.Height);
    Dest := Rect(0, Margin, FLed.Width, FLed.Height+Margin);
    Brush.Color := Self.Color;
    BrushCopy(Dest, FLed, Src, clBlack);
   end;
end;

procedure TOLabel.DrawFocus;
begin
  with Canvas do
  begin
    Brush.Style := bsClear;
    Font := Self.Font;
    with Pen do
    begin
      if Focused then Color := clBlack else Color := Parent.Brush.Color;
      Width := 1;
      Style := psDot;
    end;
    FCaptRect.Left := 18+(Margin-2);
    FCaptRect.Top := 5;
    InflateRect(FCaptRect, 2, 2);
    Rectangle(FCaptRect.Left, FCaptRect.Top, FCaptRect.Right, FCaptRect.Bottom);
  end;
end;

procedure TOLabel.DrawCaption;
var
  uFormat: UINT;
  DrawTextParams: TDrawTextParams;
begin
  with Canvas do
  begin
    Font := Self.Font;
    if not Enabled then Font.Color := clGray;
    FCaptRect := Rect(17+Margin, Margin, ClientWidth-4, ClientHeight-Margin);

    DrawTextParams.iTabLength := TabSize;
    DrawTextParams.iLeftMargin := MarginLeft;
    DrawTextParams.iRightMargin := MarginRight;
    DrawTextParams.cbSize := SizeOf(DrawTextParams);

    uFormat := DT_CALCRECT or DT_WORDBREAK or DT_EXPANDTABS or DT_END_ELLIPSIS;
    case Alignment of
      taLeftJustify  : uFormat := uFormat or DT_LEFT;
      taRightJustify : uFormat := uFormat or DT_RIGHT;
      taCenter       : uFormat := uFormat or DT_CENTER;
    end;
    DrawTextEx(Handle, PChar(Caption), Length(Caption), FCaptRect, uFormat, @DrawTextParams);
    uFormat := DT_WORDBREAK or DT_EXPANDTABS or DT_END_ELLIPSIS or DT_TABSTOP;
    case Alignment of
      taLeftJustify  : uFormat := uFormat or DT_LEFT;
      taRightJustify : uFormat := uFormat or DT_RIGHT;
      taCenter       : uFormat := uFormat or DT_CENTER;
    end;

    if FCaptRect.Bottom+FCaptRect.Top > ClientHeight then FCaptRect.Bottom := ClientHeight-FCaptRect.Top;
    DrawTextEx(Handle, PChar(Caption), Length(Caption), FCaptRect, uFormat, @DrawTextParams);
  end;
end;

procedure TOLabel.KeyDown(var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_RETURN) or (Key = VK_SPACE) then
  begin
    HasMouse := true;
    MouseIsDown := true;
    Repaint;
  end;
  inherited KeyDown(Key, Shift);
end;

procedure TOLabel.KeyUp(var Key: Word; Shift: TShiftState);
var
  CurPos: TPoint;
begin
  if (Key = VK_RETURN) or (KEY = VK_SPACE) then
  begin
    GetCursorPos(CurPos);
    CurPos := ScreenToClient(CurPos);
    if PtInRect(GetClientRect, CurPos) then HasMouse := true else HasMouse := false;
    MouseIsDown := false;
    Repaint;
  end;
  inherited KeyUp(Key, Shift);
end;

procedure TOLabel.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if (Button = mbLeft) and Enabled then
  begin
    SetFocus;
    MouseIsDown := true;
    Repaint;
    inherited MouseDown(Button, Shift, X, Y);
  end;
end;

procedure TOLabel.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if (Button = mbLeft) and Enabled then
  begin
    MouseIsDown := false;
    Repaint;
    inherited MouseUp(Button, Shift, X, Y);
  end;
end;

procedure TOLabel.Paint;
begin
  inherited Paint;
  if (csDesigning in Componentstate) or Visible then
  begin
    DrawLed;
    DrawCaption;
    DrawFocus;
  end;
end;

procedure TOLabel.DoEnter;
begin
  inherited DoEnter;
  Focused := true;
  Repaint;
end;

procedure TOLabel.DoExit;
begin
  inherited DoExit;
  Focused := false;
  Repaint;
end;

constructor TOLabel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FLed := TBitmap.Create;
  FLed.Handle := LoadBitmap(hInstance, 'OFFRND');
  if not (csDesigning in ComponentState) then controlstyle := controlstyle + [csOpaque, csSetCaption];
  SetBounds(0, 0, 100, 21);
  FAlignment := taLeftJustify;
  LedType := ltDefault;
  MarginLeft := 1;
  MarginRight := 1;
  TabSize := 3;
  TabStop := true;
end;

destructor TOLabel.Destroy;
begin
  FLed.Free;
  inherited Destroy;
end;

end.
